@@ -244,6 +244,15 @@ static int setup_opts_from_req(int sk, CriuOpts *req)
 	if (req->root)
 		opts.root = req->root;
 
+	if (req->has_rst_sibling) {
+		if (!opts.swrk_restore) {
+			pr_err("rst_sibling is not allowed in standalone service\n");
+			return -1;
+		}
+
+		opts.restore_sibling = req->rst_sibling;
+	}
+
 	if (req->has_tcp_established)
 		opts.tcp_established_ok = req->tcp_established;
 
