@@ -14,9 +14,9 @@
 
 #include "compiler.h"
 #include "asm/types.h"
+#include "fdset.h"
 #include "eventfd.h"
 #include "proc_parse.h"
-#include "crtools.h"
 #include "image.h"
 #include "util.h"
 #include "log.h"
@@ -33,9 +33,9 @@ struct eventfd_file_info {
 };
 
 /* Checks if file descriptor @lfd is eventfd */
-int is_eventfd_link(int lfd)
+int is_eventfd_link(char *link)
 {
-	return is_anon_link_type(lfd, "[eventfd]");
+	return is_anon_link_type(link, "[eventfd]");
 }
 
 static void pr_info_eventfd(char *action, EventfdFileEntry *efe)
