@@ -65,10 +65,11 @@ struct cr_fd_desc_tmpl fdset_template[CR_FD_MAX] = {
 	FD_ENTRY(IFADDR,	"ifaddr-%d"),
 	FD_ENTRY(ROUTE,		"route-%d"),
 	FD_ENTRY(IPTABLES,	"iptables-%d"),
-	FD_ENTRY(TMPFS,		"tmpfs-%d.tar.gz"),
+	FD_ENTRY(TMPFS_IMG,	"tmpfs-%d.tar.gz"),
+	FD_ENTRY(TMPFS_DEV,	"tmpfs-dev-%d.tar.gz"),
 	FD_ENTRY(TTY_FILES,	"tty"),
 	FD_ENTRY(TTY_INFO,	"tty-info"),
-	FD_ENTRY(FILE_LOCKS,	"filelocks-%d"),
+	FD_ENTRY(FILE_LOCKS,	"filelocks"),
 	FD_ENTRY(RLIMIT,	"rlimit-%d"),
 	FD_ENTRY(PAGES,		"pages-%u"),
 	FD_ENTRY(PAGES_OLD,	"pages-%d"),
@@ -76,6 +77,8 @@ struct cr_fd_desc_tmpl fdset_template[CR_FD_MAX] = {
 	FD_ENTRY(SIGNAL,	"signal-s-%d"),
 	FD_ENTRY(PSIGNAL,	"signal-p-%d"),
 	FD_ENTRY(TUNFILE,	"tunfile"),
+	FD_ENTRY(CGROUP,	"cgroup"),
+	FD_ENTRY(TIMERFD,	"timerfd"),
 
 	[CR_FD_STATS] = {
 		.fmt	= "stats-%s",
@@ -86,4 +89,9 @@ struct cr_fd_desc_tmpl fdset_template[CR_FD_MAX] = {
 		.fmt	= "irmap-cache",
 		.magic	= IRMAP_CACHE_MAGIC,
 	},
+
+	[CR_FD_FILE_LOCKS_PID] = {
+		.fmt	= "filelocks-%d.img",
+		.magic	= FILE_LOCKS_MAGIC,
+	},
 };
