@@ -13,6 +13,18 @@ struct script {
 
 #define SCRIPT_RPC_NOTIFY	(char *)0x1
 
+/*
+ * CPU capability options.
+ */
+#define CPU_CAP_FPU		(1u)
+#define CPU_CAP_ALL		(-1u)
+
+struct cg_root_opt {
+	struct list_head node;
+	char *controller;
+	char *newroot;
+};
+
 struct cr_options {
 	int			final_state;
 	char			*show_dump_file;
@@ -20,6 +32,7 @@ struct cr_options {
 	bool			check_ms_kernel;
 	bool			show_pages_content;
 	bool			restore_detach;
+	bool			restore_sibling;
 	bool			ext_unix_sk;
 	bool			shell_job;
 	bool			handle_file_locks;
@@ -28,11 +41,13 @@ struct cr_options {
 	bool			link_remap_ok;
 	unsigned int		rst_namespaces_flags;
 	bool			log_file_per_pid;
+	bool			swrk_restore;
 	char			*output;
 	char			*root;
 	char			*pidfile;
 	struct list_head	veth_pairs;
 	struct list_head	scripts;
+	struct list_head	ext_mounts;
 	char			*libdir;
 	bool			use_page_server;
 	unsigned short		ps_port;
@@ -40,6 +55,13 @@ struct cr_options {
 	bool			track_mem;
 	char			*img_parent;
 	bool			auto_dedup;
+	unsigned int		cpu_cap;
+	bool			force_irmap;
+	char			**exec_cmd;
+	bool			manage_cgroups;
+	char			*new_global_cg_root;
+	struct list_head	new_cgroup_roots;
+	bool			aufs;		/* auto-deteced, not via cli */
 };
 
 extern struct cr_options opts;
