@@ -72,10 +72,13 @@
 #define NETLINK_SK_MAGIC	0x58005614 /* Perm */
 #define NS_FILES_MAGIC		0x61394011 /* Nyandoma */
 #define TUNFILE_MAGIC		0x57143751 /* Kalyazin */
+#define CGROUP_MAGIC		0x59383330 /* Tikhvin */
+#define TIMERFD_MAGIC		0x50493712 /* Korocha */
 
 #define IFADDR_MAGIC		RAW_IMAGE_MAGIC
 #define ROUTE_MAGIC		RAW_IMAGE_MAGIC
-#define TMPFS_MAGIC		RAW_IMAGE_MAGIC
+#define TMPFS_IMG_MAGIC		RAW_IMAGE_MAGIC
+#define TMPFS_DEV_MAGIC		RAW_IMAGE_MAGIC
 #define IPTABLES_MAGIC		RAW_IMAGE_MAGIC
 
 #define PAGES_OLD_MAGIC		PAGEMAP_MAGIC
