@@ -4,6 +4,7 @@
 #include "asm/types.h"
 #include "pid.h"
 #include "list.h"
+#include "config.h"
 
 #define BUILTIN_SYSCALL_SIZE	8
 
@@ -56,10 +57,11 @@ struct parasite_ctl {
 };
 
 extern int parasite_dump_sigacts_seized(struct parasite_ctl *ctl, struct cr_fdset *cr_fdset);
-extern int parasite_dump_itimers_seized(struct parasite_ctl *ctl, struct cr_fdset *cr_fdset);
+extern int parasite_dump_itimers_seized(struct parasite_ctl *ctl, struct pstree_item *);
 
 struct proc_posix_timers_stat;
-extern int parasite_dump_posix_timers_seized(struct proc_posix_timers_stat *proc_args, struct parasite_ctl *ctl, struct cr_fdset *cr_fdset);
+extern int parasite_dump_posix_timers_seized(struct proc_posix_timers_stat *proc_args,
+		struct parasite_ctl *ctl, struct pstree_item *);
 
 #define parasite_args(ctl, type)					\
 	({								\
@@ -117,8 +119,10 @@ extern int __parasite_execute_syscall(struct parasite_ctl *ctl,
 				user_regs_struct_t *regs);
 extern bool arch_can_dump_task(pid_t pid);
 
+#ifdef CONFIG_VFDO
 extern int parasite_fixup_vdso(struct parasite_ctl *ctl, pid_t pid,
 			       struct vm_area_list *vma_area_list);
+#endif
 
 extern int parasite_stop_on_syscall(int tasks, int sys_nr);
 extern int parasite_unmap(struct parasite_ctl *ctl, unsigned long addr);
