@@ -9,6 +9,7 @@
 #include "log.h"
 #include "bug.h"
 #include "kerndat.h"
+#include "fs-magic.h"
 #include "mem.h"
 #include "compiler.h"
 #include "sysctl.h"
@@ -52,6 +53,32 @@ static int kerndat_get_shmemdev(void)
 	return 0;
 }
 
+struct stat *kerndat_get_devpts_stat()
+{
+	static struct stat st = {};
+	struct statfs fst;
+
+	if (st.st_dev != 0)
+		return &st;
+
+	if (statfs("/dev/pts", &fst)) {
+		pr_perror("Unable to statefs /dev/pts");
+		return NULL;
+	}
+	if (fst.f_type != DEVPTS_SUPER_MAGIC) {
+		pr_err("devpts isn't mount on the host\n");
+		return NULL;
+	}
+
+	/* The root /dev/pts is mounted w/o newinstance, isn't it? */
+	if (stat("/dev/pts", &st)) {
+		pr_perror("Unable to stat /dev/pts");
+		return NULL;
+	}
+
+	return &st;
+}
+
 /*
  * Check whether pagemap reports soft dirty bit. Kernel has
  * this functionality under CONFIG_MEM_SOFT_DIRTY option.
@@ -181,19 +208,6 @@ static int init_zero_page_pfn()
 	return ret;
 }
 
-int kerndat_init(void)
-{
-	int ret;
-
-	ret = kerndat_get_shmemdev();
-	if (!ret)
-		ret = kerndat_get_dirty_track();
-	if (!ret)
-		ret = init_zero_page_pfn();
-
-	return ret;
-}
-
 int kern_last_cap;
 
 int get_last_cap(void)
@@ -206,6 +220,21 @@ int get_last_cap(void)
 	return sysctl_op(req, CTL_READ);
 }
 
+int kerndat_init(void)
+{
+	int ret;
+
+	ret = kerndat_get_shmemdev();
+	if (!ret)
+		ret = kerndat_get_dirty_track();
+	if (!ret)
+		ret = init_zero_page_pfn();
+	if (!ret)
+		ret = get_last_cap();
+
+	return ret;
+}
+
 int kerndat_init_rst(void)
 {
 	int ret;
