@@ -59,20 +59,31 @@ int do_task_reset_dirty_track(int pid)
 
 unsigned int dump_pages_args_size(struct vm_area_list *vmas)
 {
-	/*
-	 * In the worst case I need one iovec for half of the
-	 * pages (e.g. every odd/even)
-	 */
-
+	/* In the worst case I need one iovec for each page */
 	return sizeof(struct parasite_dump_pages_args) +
 		vmas->nr * sizeof(struct parasite_vma_entry) +
-		(vmas->priv_size + 1) * sizeof(struct iovec) / 2;
+		(vmas->priv_size + 1) * sizeof(struct iovec);
 }
 
 static inline bool should_dump_page(VmaEntry *vmae, u64 pme)
 {
+#ifdef CONFIG_VDSO
+	/*
+	 * vDSO area must be always dumped because on restore
+	 * we might need to generate a proxy.
+	 */
 	if (vma_entry_is(vmae, VMA_AREA_VDSO))
 		return true;
+	/*
+	 * In turn VVAR area is special and referenced from
+	 * vDSO area by IP addressing (at least on x86) thus
+	 * never ever dump its content but always use one provided
+	 * by the kernel on restore, ie runtime VVAR area must
+	 * be remapped into proper place..
+	 */
+	if (vma_entry_is(vmae, VMA_AREA_VVAR))
+		return false;
+#endif
 	/*
 	 * Optimisation for private mapping pages, that haven't
 	 * yet being COW-ed
@@ -106,7 +117,7 @@ static inline bool page_in_parent(u64 pme)
  * the memory contents is present in the pagent image set.
  */
 
-static int generate_iovs(struct vma_area *vma, struct page_pipe *pp, u64 *map, u64 *off)
+static int generate_iovs(struct vma_area *vma, struct page_pipe *pp, u64 *map, u64 *off, bool has_parent)
 {
 	u64 *at = &map[PAGE_PFN(*off)];
 	unsigned long pfn, nr_to_scan;
@@ -130,7 +141,7 @@ static int generate_iovs(struct vma_area *vma, struct page_pipe *pp, u64 *map, u
 		 * page. The latter would be checked in page-xfer.
 		 */
 
-		if (page_in_parent(at[pfn])) {
+		if (has_parent && page_in_parent(at[pfn])) {
 			ret = page_pipe_add_hole(pp, vaddr);
 			pages[0]++;
 		} else {
@@ -256,8 +267,8 @@ static int __parasite_dump_pages_seized(struct parasite_ctl *ctl,
 		return -1;
 
 	ret = -1;
-	pp = create_page_pipe(vma_area_list->priv_size / 2,
-				pargs_iovs(args), pp_ret == NULL);
+	pp = create_page_pipe(vma_area_list->priv_size,
+			      pargs_iovs(args), pp_ret == NULL);
 	if (!pp)
 		goto out;
 
@@ -282,7 +293,7 @@ static int __parasite_dump_pages_seized(struct parasite_ctl *ctl,
 		if (!map)
 			goto out_xfer;
 again:
-		ret = generate_iovs(vma_area, pp, map, &off);
+		ret = generate_iovs(vma_area, pp, map, &off, xfer.parent);
 		if (ret == -EAGAIN) {
 			BUG_ON(pp_ret);
 
@@ -376,9 +387,9 @@ int prepare_mm_pid(struct pstree_item *i)
 	int fd, ret = -1, vn = 0;
 	struct rst_info *ri = i->rst;
 
-	fd = open_image(CR_FD_MM, O_RSTR, pid);
+	fd = open_image(CR_FD_MM, O_RSTR | O_OPT, pid);
 	if (fd < 0) {
-		if (errno == ENOENT)
+		if (fd == -ENOENT)
 			return 0;
 		return -1;
 	}
