@@ -21,6 +21,18 @@ struct page_server_iov {
 	u64	dst_id;
 };
 
+static void psi2iovec(struct page_server_iov *ps, struct iovec *iov)
+{
+	iov->iov_base = decode_pointer(ps->vaddr);
+	iov->iov_len = ps->nr_pages * PAGE_SIZE;
+}
+
+static void iovec2psi(struct iovec *iov, struct page_server_iov *ps)
+{
+	ps->vaddr = encode_pointer(iov->iov_base);
+	ps->nr_pages = iov->iov_len / PAGE_SIZE;
+}
+
 static int open_page_local_xfer(struct page_xfer *xfer, int fd_type, long id);
 
 #define PS_IOV_ADD	1
@@ -102,9 +114,7 @@ static int page_server_add(int sk, struct page_server_iov *pi)
 	if (prep_loc_xfer(pi))
 		return -1;
 
-	iov.iov_base = decode_pointer(pi->vaddr);
-	iov.iov_len = pi->nr_pages * PAGE_SIZE;
-
+	psi2iovec(pi, &iov);
 	if (lxfer->write_pagemap(lxfer, &iov))
 		return -1;
 
@@ -141,9 +151,7 @@ static int page_server_hole(int sk, struct page_server_iov *pi)
 	if (prep_loc_xfer(pi))
 		return -1;
 
-	iov.iov_base = decode_pointer(pi->vaddr);
-	iov.iov_len = pi->nr_pages * PAGE_SIZE;
-
+	psi2iovec(pi, &iov);
 	if (lxfer->write_hole(lxfer, &iov))
 		return -1;
 
@@ -167,7 +175,7 @@ static int page_server_serve(int sk)
 	while (1) {
 		struct page_server_iov pi;
 
-		ret = read(sk, &pi, sizeof(pi));
+		ret = recv(sk, &pi, sizeof(pi), MSG_WAITALL);
 		if (!ret)
 			break;
 
@@ -216,7 +224,7 @@ static int page_server_serve(int sk)
 			break;
 	}
 
-	if (!flushed) {
+	if (!ret && !flushed) {
 		pr_err("The data were not flushed\n");
 		ret = -1;
 	}
@@ -381,8 +389,7 @@ static int write_pagemap_to_server(struct page_xfer *xfer,
 
 	pi.cmd = PS_IOV_ADD;
 	pi.dst_id = xfer->dst_id;
-	pi.vaddr = encode_pointer(iov->iov_base);
-	pi.nr_pages = iov->iov_len / PAGE_SIZE;
+	iovec2psi(iov, &pi);
 
 	if (write(xfer->fd, &pi, sizeof(pi)) != sizeof(pi)) {
 		pr_perror("Can't write pagemap to server");
@@ -411,8 +418,7 @@ static int write_hole_to_server(struct page_xfer *xfer, struct iovec *iov)
 
 	pi.cmd = PS_IOV_HOLE;
 	pi.dst_id = xfer->dst_id;
-	pi.vaddr = encode_pointer(iov->iov_base);
-	pi.nr_pages = iov->iov_len / PAGE_SIZE;
+	iovec2psi(iov, &pi);
 
 	if (write(xfer->fd, &pi, sizeof(pi)) != sizeof(pi)) {
 		pr_perror("Can't write pagehole to server");
@@ -457,10 +463,8 @@ static int write_pagemap_loc(struct page_xfer *xfer,
 	int ret;
 	PagemapEntry pe = PAGEMAP_ENTRY__INIT;
 
-	pe.vaddr = encode_pointer(iov->iov_base);
-	pe.nr_pages = iov->iov_len / PAGE_SIZE;
-
-	if (opts.auto_dedup && !opts.use_page_server && xfer->parent != NULL) {
+	iovec2pagemap(iov, &pe);
+	if (opts.auto_dedup && xfer->parent != NULL) {
 		ret = dedup_one_iovec(xfer->parent, iov);
 		if (ret == -1) {
 			pr_perror("Auto-deduplication failed");
@@ -544,8 +548,7 @@ static int write_pagehole_loc(struct page_xfer *xfer, struct iovec *iov)
 		}
 	}
 
-	pe.vaddr = encode_pointer(iov->iov_base);
-	pe.nr_pages = iov->iov_len / PAGE_SIZE;
+	iovec2pagemap(iov, &pe);
 	pe.has_in_parent = true;
 	pe.in_parent = true;
 
@@ -656,9 +659,9 @@ static int open_page_local_xfer(struct page_xfer *xfer, int fd_type, long id)
 			return -1;
 		}
 
-		ret = open_page_read_at(pfd, id, xfer->parent, O_RDWR);
+		ret = open_page_read_at(pfd, id, xfer->parent, O_RDWR, false);
 		if (ret) {
-			pr_perror("Can't dedup old image format");
+			pr_perror("No parent image found, though parent directory is set");
 			xfree(xfer->parent);
 			xfer->parent = NULL;
 			close(pfd);
