@@ -29,26 +29,33 @@
 
 #define INET_ADDR_LEN		40
 
-struct pb_pr_field_s {
-	void *data;
-	int number;
-	int depth;
-	int count;
-	char fmt[32];
-};
+typedef struct {
+	void		*data;
+	int		number;
+	int		depth;
+	int		count;
+	char		fmt[32];
+} pb_pr_field_t;
 
-typedef struct pb_pr_field_s pb_pr_field_t;
+typedef struct {
+	void		*arg;
+	int		single_entry;
+	const char	*pretty_fmt;
+	pb_pr_field_t	cur;
+} pb_pr_ctl_t;
 
-struct pb_pr_ctrl_s {
-	void *arg;
-	int single_entry;
-	const char *pretty_fmt;
-	pb_pr_field_t cur;
-};
-
-typedef struct pb_pr_ctrl_s pb_pr_ctl_t;
 typedef int (*pb_pr_show_t)(pb_pr_field_t *field);
 
+/*
+ * This one describes how fields should be shown
+ * @fsize is the size of the field entry
+ * @show is the callback to print the entry
+ */
+struct pb_shower {
+	size_t fsize;
+	pb_pr_show_t show;
+};
+
 static int pb_msg_int32x(pb_pr_field_t *field)
 {
 	pr_msg("%#x", *(int *)field->data);
@@ -96,8 +103,7 @@ static inline void print_tabs(pb_pr_ctl_t *ctl)
 
 static void print_nested_message_braces(pb_pr_ctl_t *ctl, int right_brace)
 {
-	if (right_brace)
-		print_tabs(ctl);
+	print_tabs(ctl);
 	pr_msg("%s%s", (right_brace) ? "}" : "{", (ctl->single_entry) ? "\n" : " ");
 }
 
@@ -106,12 +112,14 @@ static void pb_show_msg(const void *msg, pb_pr_ctl_t *ctl);
 static int show_nested_message(pb_pr_field_t *field)
 {
 	pb_pr_ctl_t *ctl = container_of(field, pb_pr_ctl_t, cur);
+	void *arg = ctl->arg;
 
 	print_nested_message_braces(ctl, 0);
 	field->depth++;
 	pb_show_msg(field->data, ctl);
 	field->depth--;
 	print_nested_message_braces(ctl, 1);
+	ctl->arg = arg;
 	return 0;
 }
 
@@ -157,47 +165,6 @@ static int show_bytes(pb_pr_field_t *field)
 	return 0;
 }
 
-static size_t pb_show_prepare_field_context(const ProtobufCFieldDescriptor *fd,
-					  pb_pr_ctl_t *ctl)
-{
-	size_t fsize = 0;
-
-	switch (fd->type) {
-	case PROTOBUF_C_TYPE_ENUM:
-		ctl->arg = (void *)fd->descriptor;
-	case PROTOBUF_C_TYPE_INT32:
-	case PROTOBUF_C_TYPE_SINT32:
-	case PROTOBUF_C_TYPE_UINT32:
-	case PROTOBUF_C_TYPE_SFIXED32:
-	case PROTOBUF_C_TYPE_FLOAT:
-		fsize = 4;
-		break;
-	case PROTOBUF_C_TYPE_INT64:
-	case PROTOBUF_C_TYPE_SINT64:
-	case PROTOBUF_C_TYPE_SFIXED64:
-	case PROTOBUF_C_TYPE_FIXED32:
-	case PROTOBUF_C_TYPE_UINT64:
-	case PROTOBUF_C_TYPE_FIXED64:
-	case PROTOBUF_C_TYPE_DOUBLE:
-		fsize = 8;
-		break;
-	case PROTOBUF_C_TYPE_MESSAGE:
-		ctl->arg = (void *)fd->descriptor;
-	case PROTOBUF_C_TYPE_STRING:
-		fsize = sizeof (void *);
-		break;
-	case PROTOBUF_C_TYPE_BOOL:
-		fsize = sizeof (protobuf_c_boolean);
-		break;
-	case PROTOBUF_C_TYPE_BYTES:
-		fsize = sizeof (ProtobufCBinaryData);
-		break;
-	default:
-		BUG();
-	}
-	return fsize;
-}
-
 static int pb_show_pretty(pb_pr_field_t *field)
 {
 	switch (field->fmt[0]) {
@@ -304,54 +271,75 @@ static int pb_field_show_pretty(const ProtobufCFieldDescriptor *fd, pb_pr_ctl_t
 	return 0;
 }
 
-static pb_pr_show_t get_pb_show_function(int type, int label)
+static void pb_prepare_shower(const ProtobufCFieldDescriptor *fd,
+		pb_pr_ctl_t *ctl, struct pb_shower *sh)
 {
-	switch (type) {
+	sh->fsize = 0;
+	sh->show = pb_msg_unk;
+
+	switch (fd->type) {
 	case PROTOBUF_C_TYPE_INT32:
 	case PROTOBUF_C_TYPE_SINT32:
 	case PROTOBUF_C_TYPE_UINT32:
 	case PROTOBUF_C_TYPE_SFIXED32:
-		return pb_msg_int32x;
+		sh->fsize = 4;
+		sh->show = pb_msg_int32x;
+		break;
+
 	case PROTOBUF_C_TYPE_INT64:
 	case PROTOBUF_C_TYPE_SINT64:
 	case PROTOBUF_C_TYPE_SFIXED64:
 	case PROTOBUF_C_TYPE_FIXED32:
 	case PROTOBUF_C_TYPE_UINT64:
 	case PROTOBUF_C_TYPE_FIXED64:
-		return (label == PROTOBUF_C_LABEL_REPEATED ?
+		sh->fsize = 8;
+		sh->show = (fd->label == PROTOBUF_C_LABEL_REPEATED ?
 				pb_msg_int64x_r : pb_msg_int64x);
-	case PROTOBUF_C_TYPE_STRING:
-		return pb_msg_string;
-	case PROTOBUF_C_TYPE_MESSAGE:
-		return show_nested_message;
-	case PROTOBUF_C_TYPE_ENUM:
-		return show_enum;
-	case PROTOBUF_C_TYPE_BOOL:
-		return show_bool;
-	case PROTOBUF_C_TYPE_BYTES:
-		return show_bytes;
-	case PROTOBUF_C_TYPE_FLOAT:
-	case PROTOBUF_C_TYPE_DOUBLE:
 		break;
+
+	case PROTOBUF_C_TYPE_STRING:
+		sh->fsize = sizeof (void *);
+		sh->show = pb_msg_string;
+		break;
+	case PROTOBUF_C_TYPE_MESSAGE:
+		sh->fsize = sizeof (void *);
+		sh->show = show_nested_message;
+		ctl->arg = (void *)fd->descriptor;
+		break;
+	case PROTOBUF_C_TYPE_ENUM:
+		sh->fsize = 4;
+		sh->show = show_enum;
+		ctl->arg = (void *)fd->descriptor;
+		break;
+
+	case PROTOBUF_C_TYPE_BOOL:
+		sh->fsize = sizeof (protobuf_c_boolean);
+		sh->show = show_bool;
+		break;
+	case PROTOBUF_C_TYPE_BYTES:
+		sh->fsize = sizeof (ProtobufCBinaryData);
+		sh->show = show_bytes;
+		break;
+	case PROTOBUF_C_TYPE_FLOAT:
+		sh->fsize = 4;
+		break;
+	case PROTOBUF_C_TYPE_DOUBLE:
+		sh->fsize = 8;
+		break;
+
 	default:
 		BUG();
 	}
-	return pb_msg_unk;
-}
 
-static pb_pr_show_t get_show_function(const ProtobufCFieldDescriptor *fd, pb_pr_ctl_t *ctl)
-{
 	if (pb_field_show_pretty(fd, ctl))
-		return pb_show_pretty;
-	return get_pb_show_function(fd->type, fd->label);
+		sh->show = pb_show_pretty;
 }
 
-static void pb_show_repeated(const ProtobufCFieldDescriptor *fd, pb_pr_ctl_t *ctl, int nr_fields, pb_pr_show_t show,
-			  size_t fsize)
+static void pb_show_repeated(const ProtobufCFieldDescriptor *fd,
+		pb_pr_ctl_t *ctl, struct pb_shower *sh)
 {
 	pb_pr_field_t *field = &ctl->cur;
-	unsigned long counter;
-	int done;
+	unsigned long i, nr_fields = field->count;
 
 	if (nr_fields == 0) {
 		pr_msg("<empty>");
@@ -361,44 +349,33 @@ static void pb_show_repeated(const ProtobufCFieldDescriptor *fd, pb_pr_ctl_t *ct
 	if (fd->type == PROTOBUF_C_TYPE_MESSAGE) {
 		void *p = field->data;
 
-		field->count = nr_fields;
-		field->data = (void *)(*(long *)p);
-		done = show(field);
-		if (done)
-			return;
-
-		for (p += fsize, counter = 0; counter < nr_fields - 1; counter++, p += fsize) {
-			pr_msg(":");
+		for (i = 0; i < nr_fields; i++) {
 			field->data = (void *)(*(long *)p);
-			show(field);
+			sh->show(field);
+			p += sh->fsize;
 		}
+
 		return;
 	}
 
-	field->count = nr_fields;
-	done = show(field);
-	if (done)
-		return;
-
-	field->data += fsize;
-
-	for (counter = 0; counter < nr_fields - 1; counter++, field->data += fsize) {
-		pr_msg(":");
-		show(field);
+	for (i = 0; i < nr_fields; i++) {
+		if (i)
+			pr_msg(":");
+		if (sh->show(field))
+			break;
+		field->data += sh->fsize;
 	}
 }
 
-static void pb_show_field(const ProtobufCFieldDescriptor *fd,
-			  int nr_fields, pb_pr_ctl_t *ctl)
+static void pb_show_field(const ProtobufCFieldDescriptor *fd, pb_pr_ctl_t *ctl)
 {
-	pb_pr_show_t show;
+	struct pb_shower sh;
 
 	print_tabs(ctl);
 	pr_msg("%s: ", fd->name);
 
-	show = get_show_function(fd, ctl);
-
-	pb_show_repeated(fd, ctl, nr_fields, show, pb_show_prepare_field_context(fd, ctl));
+	pb_prepare_shower(fd, ctl, &sh);
+	pb_show_repeated(fd, ctl, &sh);
 
 	if (ctl->single_entry)
 		pr_msg("\n");
@@ -477,8 +454,9 @@ static void pb_show_msg(const void *msg, pb_pr_ctl_t *ctl)
 
 		ctl->cur.data = data;
 		ctl->cur.number = i + 1;
+		ctl->cur.count = nr_fields;
 
-		pb_show_field(&fd, nr_fields, ctl);
+		pb_show_field(&fd, ctl);
 	}
 }
 
@@ -654,6 +632,7 @@ err:
 
 int collect_image(struct collect_image_info *cinfo)
 {
+	bool optional = !!(cinfo->flags & COLLECT_OPTIONAL);
 	int fd, ret;
 	void *(*o_alloc)(size_t size) = malloc;
 	void (*o_free)(void *ptr) = free;
@@ -661,14 +640,15 @@ int collect_image(struct collect_image_info *cinfo)
 	pr_info("Collecting %d/%d (flags %x)\n",
 			cinfo->fd_type, cinfo->pb_type, cinfo->flags);
 
-	fd = open_image(cinfo->fd_type, O_RSTR);
+	fd = open_image(cinfo->fd_type, O_RSTR | (optional ? O_OPT : 0));
 	if (fd < 0) {
-		if ((cinfo->flags & COLLECT_OPTIONAL) && (errno == ENOENT))
+		if (optional && fd == -ENOENT)
 			return 0;
 		else
 			return -1;
 	}
 
+	cinfo->flags |= COLLECT_HAPPENED;
 	if (cinfo->flags & COLLECT_SHARED) {
 		o_alloc = shmalloc;
 		o_free = shfree_last;
