@@ -4,13 +4,14 @@
 #include <linux/rtnetlink.h>
 #include <unistd.h>
 #include <string.h>
-#include "crtools.h"
 #include "asm/types.h"
+#include "fdset.h"
 #include "files.h"
 #include "sockets.h"
 #include "libnetlink.h"
 #include "sk-packet.h"
 #include "packet_diag.h"
+#include "vma.h"
 
 #include "protobuf.h"
 #include "protobuf/packet-sock.pb-c.h"
@@ -220,8 +221,8 @@ int dump_socket_map(struct vma_area *vma)
 		return -1;
 	}
 
-	pr_info("Dumping socket map %x -> %"PRIx64"\n", sd->file_id, vma->vma.start);
-	vma->vma.shmid = sd->file_id;
+	pr_info("Dumping socket map %x -> %"PRIx64"\n", sd->file_id, vma->e->start);
+	vma->e->shmid = sd->file_id;
 	return 0;
 }
 
