@@ -115,6 +115,8 @@ void write_stats(int what)
 		encode_time(TIME_FROZEN, &ds_entry.frozen_time);
 		encode_time(TIME_MEMDUMP, &ds_entry.memdump_time);
 		encode_time(TIME_MEMWRITE, &ds_entry.memwrite_time);
+		ds_entry.has_irmap_resolve = true;
+		encode_time(TIME_IRMAP_RESOLVE, &ds_entry.irmap_resolve);
 
 		ds_entry.pages_scanned = dstats->counts[CNT_PAGES_SCANNED];
 		ds_entry.pages_skipped_parent = dstats->counts[CNT_PAGES_SKIPPED_PARENT];
@@ -126,6 +128,8 @@ void write_stats(int what)
 
 		rs_entry.pages_compared = atomic_read(&rstats->counts[CNT_PAGES_COMPARED]);
 		rs_entry.pages_skipped_cow = atomic_read(&rstats->counts[CNT_PAGES_SKIPPED_COW]);
+		rs_entry.has_pages_restored = true;
+		rs_entry.pages_restored = atomic_read(&rstats->counts[CNT_PAGES_RESTORED]);
 
 		encode_time(TIME_FORK, &rs_entry.forking_time);
 		encode_time(TIME_RESTORE, &rs_entry.restore_time);
@@ -144,7 +148,7 @@ void write_stats(int what)
 int init_stats(int what)
 {
 	if (what == DUMP_STATS) {
-		dstats = xmalloc(sizeof(*dstats));
+		dstats = xzalloc(sizeof(*dstats));
 		return dstats ? 0 : -1;
 	}
 
