@@ -105,18 +105,19 @@ void test_init(int argc, char **argv)
 
 	val = getenv("ZDTM_NEWNS");
 	if (val) {
-		unsetenv("ZDTM_NEWNS");
-		ns_create(argc, argv);
-		exit(1);
-	}
+		if (!strcmp(val, "1")) {
+			ns_create(argc, argv);
+			exit(1);
+		}
 
-	val = getenv("ZDTM_EXE");
-	if (val) {
-		test_log_init(outfile, "ns");
-		redir_stdfds();
-		unsetenv("ZDTM_EXE");
-		ns_init(argc, argv);
-		exit(1);
+		if (!strcmp(val, "2")) {
+			test_log_init(outfile, "ns");
+			redir_stdfds();
+			ns_init(argc, argv);
+			exit(1);
+		}
+
+		/* "3" -- run the test */
 	}
 
 	val = getenv("ZDTM_GID");
@@ -149,6 +150,9 @@ void test_init(int argc, char **argv)
 	setup_outfile();
 	redir_stdfds();
 
+	if (getenv("ZDTM_REEXEC"))
+		goto skip_pid;
+
 	pidf = fopen(pidfile, "wx");
 	if (!pidf) {
 		err("Can't create pid file %s: %m\n", pidfile);
@@ -183,22 +187,23 @@ void test_init(int argc, char **argv)
 		_exit(0);
 	}
 
+	fclose(pidf);
+
+	if (setsid() < 0) {
+		err("Can't become session group leader: %m\n");
+		exit(1);
+	}
+
+skip_pid:
 	/* record the test pid to remember the ownership of the pidfile */
 	master_pid = getpid();
 
-	fclose(pidf);
-
 	sa.sa_handler = SIG_DFL;
 	if (sigaction(SIGCHLD, &sa, NULL)) {
 		err("Can't reset SIGCHLD handler: %m\n");
 		exit(1);
 	}
 
-	if (setsid() < 0) {
-		err("Can't become session group leader: %m\n");
-		exit(1);
-	}
-
 	srand48(time(NULL));	/* just in case we need it */
 }
 
