@@ -49,7 +49,9 @@ int main(int argc, char **argv)
 	}
 
 	if (st.st_mode != mode || st.st_rdev != dev) {
-		fail("%s is no longer the device file we had");
+		fail("%s is no longer the device file we had", filename);
+		test_msg("mode %x want %x, dev %x want %x\n",
+				st.st_mode, mode, st.st_rdev, dev);
 		goto out;
 	}
 
