@@ -60,6 +60,7 @@ static void vma_opt_str(const struct vma_area *v, char *opt)
 	opt2s(VMA_AREA_STACK, "stk");
 	opt2s(VMA_AREA_VSYSCALL, "vsys");
 	opt2s(VMA_AREA_VDSO, "vdso");
+	opt2s(VMA_AREA_VVAR, "vvar");
 	opt2s(VMA_FORCE_READ, "frd");
 	opt2s(VMA_AREA_HEAP, "heap");
 	opt2s(VMA_FILE_PRIVATE, "fp");
@@ -160,7 +161,7 @@ int move_img_fd(int *img_fd, int want_fd)
 	return 0;
 }
 
-static pid_t open_proc_pid = 0;
+static pid_t open_proc_pid = PROC_NONE;
 static int open_proc_fd = -1;
 
 int close_pid_proc(void)
@@ -171,7 +172,7 @@ int close_pid_proc(void)
 		ret = close(open_proc_fd);
 
 	open_proc_fd = -1;
-	open_proc_pid = 0;
+	open_proc_pid = PROC_NONE;
 
 	return ret;
 }
@@ -228,7 +229,18 @@ inline int open_pid_proc(pid_t pid)
 		dfd = get_service_fd(PROC_FD_OFF);
 	}
 
-	snprintf(path, sizeof(path), "%d", pid);
+	if (pid == PROC_GEN)
+		/*
+		 * Don't cache it, close_pid_proc() would
+		 * close service descriptor otherwise.
+		 */
+		return dfd;
+
+	if (pid == PROC_SELF)
+		snprintf(path, sizeof(path), "self");
+	else
+		snprintf(path, sizeof(path), "%d", pid);
+
 	fd = openat(dfd, path, O_RDONLY);
 	if (fd < 0)
 		pr_perror("Can't open %s", path);
@@ -302,7 +314,7 @@ int install_service_fd(enum sfd_type type, int fd)
 
 	BUG_ON((int)type <= SERVICE_FD_MIN || (int)type >= SERVICE_FD_MAX);
 
-	if (dup2(fd, sfd) != sfd) {
+	if (dup3(fd, sfd, O_CLOEXEC) != sfd) {
 		pr_perror("Dup %d -> %d failed", fd, sfd);
 		return -1;
 	}
@@ -448,6 +460,7 @@ int run_scripts(char *action)
 {
 	struct script *script;
 	int ret = 0;
+	char image_dir[PATH_MAX];
 
 	pr_debug("Running %s scripts\n", action);
 
@@ -456,6 +469,12 @@ int run_scripts(char *action)
 		return -1;
 	}
 
+	sprintf(image_dir, "/proc/%ld/fd/%d", (long) getpid(), get_service_fd(IMG_FD_OFF));
+	if (setenv("CRTOOLS_IMAGE_DIR", image_dir, 1)) {
+		pr_perror("Can't set CRTOOLS_IMAGE_DIR=%s", image_dir);
+		return -1;
+	}
+
 	list_for_each_entry(script, &opts.scripts, node) {
 		if (script->path == SCRIPT_RPC_NOTIFY) {
 			pr_debug("\tRPC\n");
@@ -481,47 +500,6 @@ int run_scripts(char *action)
 		ret__;						\
 	})
 
-static int fixup_env(void)
-{
-	char *ep, *nep;
-	int len;
-
-	/*
-	 * FIXME
-	 *
-	 * We sometimes call other tools inside namespace
-	 * we restore. Host system may not have PATH properly
-	 * reflecting where the tools are in other namespaces.
-	 *
-	 * E.g. when restoring Centos6 container from FC20
-	 * host we fail to find /bin/tar, as in FC20 /bin is
-	 * not in PATH.
-	 *
-	 * It's not cool at all, things we think we call might
-	 * not coincide with what is really called. Proper fix
-	 * will come a bit later.
-	 */
-
-	ep = getenv("PATH");
-	if (!ep)
-		return 0;
-
-	len = strlen(ep);
-	nep = xmalloc(len + sizeof("/bin:"));
-	if (!nep)
-		return -1;
-
-	sprintf(nep, "/bin:%s", ep);
-	if (setenv("PATH", nep, 1)) {
-		pr_perror("Failed to override PATH");
-		xfree(nep);
-		return -1;
-	}
-
-	xfree(nep);
-	return 0;
-}
-
 /*
  * If "in" is negative, stdin will be closed.
  * If "out" or "err" are negative, a log file descriptor will be used.
@@ -544,9 +522,6 @@ int cr_system(int in, int out, int err, char *cmd, char *const argv[])
 		pr_perror("fork() failed");
 		goto out;
 	} else if (pid == 0) {
-		if (fixup_env())
-			goto out_chld;
-
 		if (out < 0)
 			out = log_get_fd();
 		if (err < 0)
@@ -641,6 +616,10 @@ int cr_daemon(int nochdir, int noclose)
 		int fd;
 
 		fd = open("/dev/null", O_RDWR);
+		if (fd < 0) {
+			pr_perror("Can't open /dev/null");
+			return -1;
+		}
 		dup2(fd, 0);
 		dup2(fd, 1);
 		dup2(fd, 2);
@@ -671,7 +650,7 @@ int vaddr_to_pfn(unsigned long vaddr, u64 *pfn)
 
 	off = (vaddr / PAGE_SIZE) * sizeof(u64);
 	if (lseek(fd, off, SEEK_SET) != off) {
-		pr_perror("Failed to seek address %lx\n", vaddr);
+		pr_perror("Failed to seek address %lx", vaddr);
 		goto out;
 	}
 
@@ -706,3 +685,127 @@ struct vma_area *alloc_vma_area(void)
 
 	return p;
 }
+
+int mkdirp(const char *path)
+{
+	size_t i;
+	char made_path[PATH_MAX], *pos;
+
+	if (strlen(path) >= PATH_MAX) {
+		pr_err("path %s is longer than PATH_MAX", path);
+		return -1;
+	}
+
+	strcpy(made_path, path);
+
+	i = 0;
+	if (made_path[0] == '/')
+		i++;
+
+	for (; i < strlen(made_path); i++) {
+		pos = strchr(made_path + i, '/');
+		if (pos)
+			*pos = '\0';
+		if (mkdir(made_path, 0755) < 0 && errno != EEXIST) {
+			pr_perror("couldn't mkdirpat directory");
+			return -1;
+		}
+		if (pos) {
+			*pos = '/';
+			i = pos - made_path;
+		} else
+			break;
+	}
+
+	return 0;
+}
+
+bool is_path_prefix(const char *path, const char *prefix)
+{
+	if (strstartswith(path, prefix)) {
+		size_t len = strlen(prefix);
+		switch (path[len]) {
+		case '\0':
+		case '/':
+			return true;
+		}
+	}
+
+	return false;
+}
+
+FILE *fopenat(int dirfd, char *path, char *cflags)
+{
+	int tmp, flags = 0;
+	char *iter;
+
+	for (iter = cflags; *iter; iter++) {
+		switch (*iter) {
+		case 'r':
+			flags |= O_RDONLY;
+			break;
+		case 'a':
+			flags |= O_APPEND;
+			break;
+		case 'w':
+			flags |= O_WRONLY | O_CREAT;
+			break;
+		case '+':
+			flags = O_RDWR | O_CREAT;
+			break;
+		}
+	}
+
+	tmp = openat(dirfd, path, flags, S_IRUSR | S_IWUSR);
+	if (tmp < 0)
+		return NULL;
+
+	return fdopen(tmp, cflags);
+}
+
+void split(char *str, char token, char ***out, int *n)
+{
+	int i;
+	char *cur;
+
+	*n = 0;
+	for (cur = str; cur != NULL; cur = strchr(cur, token)) {
+		(*n)++;
+		cur++;
+	}
+
+
+	*out = xmalloc((*n) * sizeof(char *));
+	if (!*out) {
+		*n = -1;
+		return;
+
+	}
+
+	cur = str;
+	i = 0;
+	do {
+		char *prev = cur;
+		cur = strchr(cur, token);
+
+		if (cur)
+			*cur = '\0';
+		(*out)[i] = xstrdup(prev);
+		if (cur) {
+			*cur = token;
+			cur++;
+		}
+
+		if (!(*out)[i]) {
+			int j;
+			for (j = 0; j < i; j++)
+				xfree((*out)[j]);
+			xfree(*out);
+			*out = NULL;
+			*n = -1;
+			return;
+		}
+
+		i++;
+	} while(cur);
+}
